#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.4'
__release__ = 20230121
__all__ = ['Time', 'Frequency']

from ..dictionaries import dictionary as _dictionary
from ..unit_class import Unit
from ..helpers.common_classes import unit_or_str, numeric


class Time(Unit):
    classUnits = _dictionary['Time']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'time'
        self.kind = Time
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)

    def __mul__(self, other):
        from .energy import Power
        if type(other) is Power:
            return super().__mul__(other).to('Wh')
        else:
            return super().__mul__(other)


class Frequency(Unit):
    classUnits = _dictionary['Frequency']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'frequency'
        self.kind = Frequency
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.3'
__release__ = 20230118
__all__ = ['Rate', 'Speed', 'Velocity', 'Acceleration']

from ..dictionaries import dictionary as _dictionary
from ..unit_class import Unit
from ..helpers.common_classes import unit_or_str, numeric


class Rate(Unit):
    classUnits = _dictionary['Rate']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'rate'
        self.kind = Rate
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)


class Velocity(Unit):
    classUnits = _dictionary['Velocity']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'velocity'
        self.kind = Velocity
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)


def Speed(value: numeric, units: unit_or_str):
    return Velocity(value, units)


class Acceleration(Unit):
    classUnits = _dictionary['Acceleration']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'acceleration'
        self.kind = Acceleration
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Aug 26 21:17:35 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.12'
__release__ = 20230121
__all__ = ['Energy', 'Power', 'Current', 'Voltage', 'Resistance', 'Conductance', 'Capacitance', 'Charge', 'Inductance', 'Impedance']

from ..dictionaries import dictionary as _dictionary
from ..unit_class import Unit
from ..helpers.common_classes import unit_or_str, numeric
from .time import Time


class Energy(Unit):
    classUnits = _dictionary['Energy']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'energy'
        self.kind = Energy
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __truediv__(self, other):
        if type(other) is Time:
            return super().__truediv__(other).to('Watt')
        if type(other) is Power:
            return super().__truediv__(other).to('hour')
        else:
            return super().__truediv__(other)


class Power(Unit):
    classUnits = _dictionary['Power']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'power'
        self.kind = Power
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __mul__(self, other):
        if type(other) is Time:
            return super().__mul__(other).to('Wh')
        else:
            return super().__mul__(other)

    def __truediv__(self, other):
        if type(other) is Current:
            return super().__truediv__(other).to('Volt')
        elif type(other) is Voltage:
            return super().__truediv__(other).to('Ampere')
        else:
            return super().__truediv__(other)


class Current(Unit):
    classUnits = _dictionary['Current']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'current'
        self.kind = Current
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __mul__(self, other):
        if type(other) is Resistance:
            return super().__mul__(other).to('Volt')
        else:
            return super().__mul__(other)


class Voltage(Unit):
    classUnits = _dictionary['Voltage']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'voltage'
        self.kind = Voltage
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __mul__(self, other):
        if type(other) is Current:
            return super().__mul__(other).to('Watt')
        if type(other) is Capacitance:
            return super().__mul__(other).to('Coulomb')
        else:
            return super().__mul__(other)

    def __truediv__(self, other):
        if type(other) is Current:
            return super().__truediv__(other).to('Ohm')
        elif type(other) is Resistance:
            return super().__truediv__(other).to('Ampere')
        else:
            return super().__truediv__(other)


class Resistance(Unit):
    classUnits = _dictionary['Resistance']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'resistance'
        self.kind = Resistance
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __mul__(self, other):
        if type(other) is Current:
            return super().__mul__(other).to('Volt')
        else:
            return super().__mul__(other)


class Conductance(Unit):
    classUnits = _dictionary['Conductance']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'conductance'
        self.kind = Conductance
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)


class Capacitance(Unit):
    classUnits = _dictionary['Capacitance']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'capacitance'
        self.kind = Capacitance
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __mul__(self, other):
        if type(other) is Voltage:
            return super().__mul__(other).to('Coulomb')
        else:
            return super().__mul__(other)


class Charge(Unit):
    classUnits = _dictionary['Charge']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'charge'
        self.kind = Charge
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)

    def __truediv__(self, other):
        if type(other) is Voltage:
            return super().__truediv__(other).to('Farad')
        else:
            return super().__truediv__(other)


class Inductance(Unit):
    classUnits = _dictionary['Inductance']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'inductance'
        self.kind = Inductance
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)


class Impedance(Unit):
    classUnits = _dictionary['Impedance']

    def __init__(self, value: numeric, unit: unit_or_str):
        self.name = 'Impedance'
        self.kind = Impedance
        self.value = self.check_value(value)
        self.unit = self.check_unit(unit)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 14:34:59 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.3'
__release__ = 20230121
__all__ = ['CustomUnits', 'UserUnits', 'OtherUnits', 'set_unit', 'set_conversion']

from ..dictionaries import dictionary
from ..unit_class import Unit
from ..helpers.common_classes import unit_or_str, numeric


def CustomUnits(value: numeric, units: unit_or_str) -> Unit:
    return UserUnits(value, units)


def OtherUnits(value: numeric, units: unit_or_str) -> Unit:
    return UserUnits(value, units)


class UserUnits(Unit):
    classUnits = dictionary['UserUnits']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'UserUnits'
        self.kind = UserUnits
        units = units.strip()
        if isinstance(units, Unit):
            units = units.unit
        if units not in dictionary['UserUnits']:
            if type(dictionary['UserUnits']) is tuple:
                dictionary['UserUnits'] = list(dictionary['UserUnits'])
            dictionary['UserUnits'].append(units)
        self.value = self.check_value(value)
        self.unit = units  # self.check_unit(units)


def set_unit(unit_name: str) -> bool:
    from ..database import units_network
    from ..network import UNode
    units_network.add_node(UNode(unit_name))


def set_conversion(from_units: str, to_units: str, conversion, reverse_conversion=None) -> bool:
    from ..database import units_network
    from ..network import UNode, Conversion
    if reverse_conversion is None:
        def reverse_conversion(x): return x / conversion(1)
    if type(from_units) is str:
        pass
    elif hasattr(from_units, 'units') and type(from_units.units) is str:
        from_units = from_units.units
    else:
        raise TypeError("`from_units` must be str or Unit.")
    if type(to_units) is str:
        pass
    elif hasattr(to_units, 'units') and type(to_units.units) is str:
        to_units = to_units.units
    else:
        raise TypeError("`to_units` must be str or Unit.")
    if not hasattr(conversion, '__call__') and hasattr(conversion, '__getitem__'):
        raise TypeError("`conversion` must be callable.")
    if not hasattr(reverse_conversion, '__call__') and hasattr(reverse_conversion, '__getitem__'):
        raise TypeError("`reverse_conversion` must be callable.")

    units_network.add_node(UNode(from_units))
    units_network.add_node(UNode(to_units))
    units_network.add_edge(Conversion(units_network.get_node(from_units),
                                      units_network.get_node(to_units),
                                      conversion))
    units_network.add_edge(Conversion(units_network.get_node(to_units),
                                      units_network.get_node(from_units),
                                      reverse_conversion))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jan 23 23:34:59 2023

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.3'
__release__ = 20230121
__all__ = ['Mass']

from ..dictionaries import dictionary as _dictionary
from ..unit_class import Unit
from ..helpers.common_classes import unit_or_str, numeric


class Mass(Unit):
    classUnits = _dictionary['Mass']

    def __init__(self, value: numeric, units: unit_or_str):
        self.name = 'mass'
        self.kind = Mass
        self.value = self.check_value(value)
        self.unit = self.check_unit(units)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 18:24:20 2020

@author: Martín Carlos Araya <martinaraya@gmail.com>
"""

__version__ = '0.5.15'
__release__ = 2023026
__all__ = ['units', 'convert', 'Unit', 'is_Unit', 'set_unit', 'set_conversion', 'convertible', 'save']

# import unyts.parameters
from .parameters import unyts_parameters_, print_path, reload, raise_error, cache
from .database import network_to_frame, save_memory, load_memory
from .units.define import units
from .converter import convert, convertible
from .unit_class import Unit, is_Unit
from .units.custom import set_unit, set_conversion

if unyts_parameters_.show_version_:
    print("loaded unyts version", __version__)
    # unyts_parameters_.show_version_ = False
    # unyts_parameters_.save_params()


def save(path=None) -> None:
    """
    Will save the current state of searches memory to a cache file that will be loaded next time unyts is started.

    Returns
    -------
    None
    """
    save_memory(path)


def load(path=None) -> None:
    """
    Will load the saved cache searches into current memory of the converter.

    Returns
    -------
    None
    """
    load_memory(path)
